/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _gdsl_bintree.h
 * $Revision: 1.5 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _gdsl_bintree.h,v 1.5 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef __GDSL_BINTREE_H_
#define __GDSL_BINTREE_H_

#include "gdsl_types.h"
#include "gdsl_macros.h"

#if defined (__cplusplus)
extern "C" {
#endif

typedef struct gdsl_bintree_node *_gdsl_bintree_t;

/*
 * Map node function type
 */
typedef int (* gdsl_bintree_map_func_t) (_gdsl_bintree_t, void *);

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a low-level binary tree.
 * Allocates a new low-level binary tree data structure which content is sets to
 * E. 
 * Its left son (resp. right) is set to LEFT (resp. RIGHT).
 * In case of success, returns the newly allocated data structure
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern _gdsl_bintree_t
_gdsl_bintree_make (const gdsl_element_t E,                    /* The content */
		    const _gdsl_bintree_t LEFT,          /* The left sub-tree */
		    const _gdsl_bintree_t RIGHT         /* The right sub-tree */
		    );

#define _gdsl_bintree_create(E) _gdsl_bintree_make ((E), NULL, NULL)

/*
 * Destroy a low-level binary tree.
 * Flush and destroy T. If FREE_F != NULL, this function is used to deallocate
 * each T's element.
 *
 * Complexity: O( |T| )
 *
 * Requires: nothing
 */
extern void 
_gdsl_bintree_destroy (_gdsl_bintree_t T,              /* The tree to destroy */
		       const gdsl_free_func_t FREE_F   /* Deallocate function */
		       );

/*
 * Copy a low-level binary tree.
 * Creates and returns a copy of T using COPY_F on each element to copy them.
 * In case of success, returns the copy of T.
 * In case of failure or if _gdsl_bintree_is_empty (T) == TRUE, returns NULL.
 *
 * Complexity: O( |T| )
 *
 * Requires: COPY_F != NULL
 */
extern _gdsl_bintree_t
_gdsl_bintree_copy (const _gdsl_bintree_t T,              /* The tree to copy */
		    const gdsl_copy_func_t COPY_F        /* The copy function */
		    );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/* 
 * Check if a low-level binary tree is empty.
 * Returns TRUE if T is empty, returns FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern bool
_gdsl_bintree_is_empty (const _gdsl_bintree_t T          /* The tree to check */
			);
  
/*
 * Check if a low-level binary tree is reduced to a leaf.
 * Returns TRUE if T is a leaf, returns FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern bool
_gdsl_bintree_is_leaf (const _gdsl_bintree_t T           /* The tree to check */
		       );

/*
 * Gets the root content of a low-level binary tree.
 * Returns the contents of T's root.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern gdsl_element_t
_gdsl_bintree_get_content (const _gdsl_bintree_t T         /* The tree to use */
			   );

/*
 * Check if a low-level binary tree is a root.
 * Returns TRUE if T is a root, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern bool
_gdsl_bintree_is_root (const _gdsl_bintree_t T           /* The tree to check */
		       );

/*
 * Gets the parent tree of a low-level binary tree.
 * Returns T's parent if T isn't a root, or NULL if T is a root.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern _gdsl_bintree_t
_gdsl_bintree_get_parent (const _gdsl_bintree_t T          /* The tree to use */
			  );

/*
 * Gets the left sub-tree of a low-level binary tree.
 * Returns T's left sub-tree.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern _gdsl_bintree_t
_gdsl_bintree_get_left_subtree (const _gdsl_bintree_t T    /* The tree to use */
				);

/*
 * Gets the right sub-tree of a low-level binary tree.
 * Returns T's right sub-tree.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern _gdsl_bintree_t
_gdsl_bintree_get_right_subtree (const _gdsl_bintree_t T   /* The tree to use */
				 );

/*
 * Gets the left sub-tree reference of a low-level binary tree.
 * Returns T's left sub-tree reference.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern _gdsl_bintree_t *
_gdsl_bintree_get_left_ref (const _gdsl_bintree_t T        /* The tree to use */
			    );

/*
 * Gets the right sub-tree reference of a low-level binary tree.
 * Returns T's right sub-tree reference.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern _gdsl_bintree_t *
_gdsl_bintree_get_right_ref (const _gdsl_bintree_t T       /* The tree to use */
			     );

/*
 * Calculate the height of a low-level binary tree.
 * Computes and returns the height of T.
 *
 * Complexity: O( |T| )
 *
 * Requires: nothing
 */
extern ulong
_gdsl_bintree_height (const _gdsl_bintree_t T  /* Tree to compute height from */
		      );

/*
 * Calculate the size of a low-level binary tree.
 * Computes ans returns the size of T (noted |T|)
 *
 * Complexity: O( |T| )
 *
 * Requires: nothing
 */
extern ulong
_gdsl_bintree_size (const _gdsl_bintree_t T      /* Tree to compute size from */
		    );

/******************************************************************************/
/* Modification functions                                                     */
/******************************************************************************/

/*
 * Sets the root element of a low-level binary tree.
 * Modify the root element of T to E.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern void
_gdsl_bintree_set_content (_gdsl_bintree_t T,           /* The tree to modify */
			   const gdsl_element_t E     /* New T's root content */
			   );

/*
 * Sets the parent tree of a low-level binary tree.
 * Modify T's parent to P.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern void
_gdsl_bintree_set_parent (_gdsl_bintree_t T,            /* The tree to modify */
			  const _gdsl_bintree_t P           /* New T's parent */
			  );

/*
 * Sets left sub-tree of a low-level binary tree.
 * Modify T's left sub-tree to L.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern void
_gdsl_bintree_set_left_subtree (_gdsl_bintree_t T,      /* The tree to modify */
				const _gdsl_bintree_t L  /* New left sub-tree */
				);
  
/*
 * Sets right sub-tree of a low-level binary tree.
 * Modify T's right sub-tree to R.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 */
extern void
_gdsl_bintree_set_right_subtree (_gdsl_bintree_t T,     /* The tree to modify */
				 const _gdsl_bintree_t R/* New right sub-tree */
				 );

/******************************************************************************/
/* Rotation functions                                                         */
/******************************************************************************/

/*
 * Left rotate a low-level binary tree.
 * Do a left rotation of T, and returns the modified T.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 *           & _gdsl_bintree_get_right_subtree (T) must be valid too
 */
extern _gdsl_bintree_t
_gdsl_bintree_left_rot (_gdsl_bintree_t *T   /* The ref of the tree to rotate */
			);

/*
 * Right rotate a low-level binary tree.
 * Do a right rotation of T, and returns the modified T.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 *           & _gdsl_bintree_get_left_subtree (T) must be valid too
 */
extern _gdsl_bintree_t
_gdsl_bintree_right_rot (_gdsl_bintree_t *T  /* The ref of the tree to rotate */
			 );

/*
 * Left-right rotate a low-level binary tree.
 * Do a double left-right rotation of T, and returns the modified T.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 *           & _gdsl_bintree_get_left_subtree (T) must be valid too
 *           & _gdsl_bintree_get_right_subtree (
 *                         _gdsl_bintree_get_left_subtree (T)) must be valid too
 */
extern _gdsl_bintree_t
_gdsl_bintree_left_right_rot (_gdsl_bintree_t *T        /* The tree to rotate */
			      );

/*
 * Right-left rotate a low-level binary tree.
 * Do a double right-left rotation of T, and returns the modified T.
 * 
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bintree_t
 *           & _gdsl_bintree_get_right_subtree (T) must be valid too
 *           & _gdsl_bintree_get_left_subtree ( 
 *                        _gdsl_bintree_get_right_subtree (T)) must be valid too
 */
extern _gdsl_bintree_t
_gdsl_bintree_right_left_rot (_gdsl_bintree_t *T        /* The tree to rotate */
			      );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse a low-level binary tree in prefixed order.
 * Parse all T's nodes in prefixed order. The MAP_F function is launched on each
 * node with the USER_DATA argument. If MAP_F returns 0, then 
 * _gdsl_bintree_prefix_parse() stops and returns its last examinated node.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_bintree_t
_gdsl_bintree_prefix_parse (const _gdsl_bintree_t T,     /* The tree to parse */
			    const gdsl_bintree_map_func_t MAP_F,/* Parse func */
			    void *USER_DATA     /* Additionnal arguments to F */
			    );

/*
 * Parse a low-level binary tree in infixed order.
 * Parse all T's nodes in infixed order. The MAP_F function is launched on each 
 * node with the USER_DATA argument. If MAP_F returns 0, then 
 * _gdsl_bintree_infix_parse() stops and returns its last examinated node.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_bintree_t
_gdsl_bintree_infix_parse (const _gdsl_bintree_t T,      /* The tree to parse */
			   const gdsl_bintree_map_func_t MAP_F, /* Parse func */
			   void *USER_DATA      /* Additionnal arguments to F */
			   );

/*
 * Parse a low-level binary tree in postfixed order.
 * Parse all T's nodes in postfixed order. The MAP_F function is launched on 
 * each node with the USER_DATA argument. If MAP_F returns 0, then 
 * _gdsl_bintree_postfix_parse() stops and returns its last examinated node.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_bintree_t
_gdsl_bintree_postfix_parse (const _gdsl_bintree_t T,    /* The tree to parse */
			     const gdsl_bintree_map_func_t MAP_F,/* Parse fun */
			     void *USER_DATA    /* Additionnal arguments to F */
			     );

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes the content of all nodes of a low-level binary tree to a file.
 * Writes T's nodes contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: WRITE_F != NULL 
 *           & OUTPUT_FILE != NULL
 */
extern void
_gdsl_bintree_write (const _gdsl_bintree_t T,            /* The tree to write */
		     const gdsl_write_func_t WRITE_F,       /* Write function */
		     FILE *OUTPUT_FILE,             /* The file to write into */
		     void *USER_DATA   /* User informations passed to WRITE_F */
		     );

/*
 * Writes the content of a low-level binary tree to a file into XML.
 * Writes T's nodes contents to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then uses WRITE_F to write T's nodes content to 
 * OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: OUTPUT_FILE != NULL
 *
 */
extern void
_gdsl_bintree_write_xml (const _gdsl_bintree_t T,        /* The tree to write */
			 const gdsl_write_func_t WRITE_F,   /* Write function */
			 FILE *OUTPUT_FILE,         /* The file to write into */
			 void *USER_DATA      /* User infos passed to WRITE_F */
			 );

/*
 * Dumps the internal structure of a low-level binary tree to a file.
 * Dumps T's structure to OUTPUT_FILE. If WRITE_F != NULL, then use WRITE_F to 
 * write nodes elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: OUTPUT_FILE != NULL
 */
extern void
_gdsl_bintree_dump (const _gdsl_bintree_t T,              /* The tree to dump */
		    const gdsl_write_func_t WRITE_F,        /* Write function */
		    FILE *OUTPUT_FILE,            /* The file where to dump T */
		    void *USER_DATA    /* User informations passed to WRITE_F */
		    );

/******************************************************************************/
/* Display functions                                                          */
/******************************************************************************/

/*
 * ONLY FOR TESTING: this function will be modified in the future.
 */
extern void 
_gdsl_bintree_gfx_display (const _gdsl_bintree_t T,
			   const char * (*key_display)(gdsl_element_t)
			   );

#ifdef __cplusplus
}
#endif

#endif /* __GDSL_BINTREE_H_ */
