/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _gdsl_list_node.h
 * $Revision: 1.5 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _gdsl_list_node.h,v 1.5 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef __GDSL_LIST_NODE_H_
#define __GDSL_LIST_NODE_H_

#include "gdsl_types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _gdsl_list_node *_gdsl_list_node_t;

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Allocates a new node.
 * Creates and allocates a new node data structure. 
 * All node's fields are sets to NULL.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern _gdsl_list_node_t
_gdsl_list_node_alloc (void);

/*
 * Deallocates a node.
 * Destroys and deallocates the previously allocated NODE.
 * The NODE's content is not modified and is returned.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern gdsl_element_t
_gdsl_list_node_free (_gdsl_list_node_t NODE        /* The node to deallocate */
		      );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/*
 * Gets the successor of a node.
 * Returns NODE's successor if NODE had a successor, NULL otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern _gdsl_list_node_t
_gdsl_list_node_get_succ (const _gdsl_list_node_t NODE     /* The node to use */
			  );

/*
 * Gets the predecessor of a node.
 * Returns NODE's predecessor if NODE had a predecessor, NULL otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern _gdsl_list_node_t
_gdsl_list_node_get_pred (const _gdsl_list_node_t NODE     /* The node to use */
			  );

/*
 * Gets the content of a node.
 * Return NODE's content if NODE had a content, NULL otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern gdsl_element_t
_gdsl_list_node_get_content (const _gdsl_list_node_t NODE  /* The node to use */
			     );

/******************************************************************************/
/* Modification functions                                                     */
/******************************************************************************/

/*
 * Sets the successor of a node.
 * Modify NODE's successor to SUCC.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern void
_gdsl_list_node_set_succ (_gdsl_list_node_t NODE,       /* The node to modify */
			  const _gdsl_list_node_t SUCC   /* Successor of NODE */
			  );

/*
 * Sets the predecessor of a node.
 * Modify NODE's predecessor to PRED.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern void
_gdsl_list_node_set_pred (_gdsl_list_node_t NODE,       /* The node to modify */
			  const _gdsl_list_node_t PRED /* Predecessor of NODE */
			  );

/*
 * Sets the content of a node.
 * Modify NODE's content to CONTENT.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern void
_gdsl_list_node_set_content (_gdsl_list_node_t NODE,    /* The node to modify */
			     const gdsl_element_t CONTENT  /* Content of NODE */
			     );

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes a node to a file.
 * Writes NODE to OUTPUT_FILE, using WRITE_F function to write its content.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 *           & WRITE_F != NULL 
 *           & OUTPUT_FILE != NULL
 */
extern void
_gdsl_list_node_write (const _gdsl_list_node_t NODE,     /* The node to write */
		       const gdsl_write_func_t WRITE_F, /* The write function */
		       FILE *OUTPUT_FILE,           /* The file to write into */
		       void *USER_DATA        /* User infos passed to WRITE_F */
		       );

/*
 * Writes a node to a file into XML.
 * Writes NODE to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then uses WRITE_F to write NODE's content.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t 
 *           & OUTPUT_FILE != NULL
 */
extern void
_gdsl_list_node_write_xml (const _gdsl_list_node_t NODE, /* The node to write */
			   const gdsl_write_func_t WRITE_F, /* Write function */
			   FILE *OUTPUT_FILE,       /* The file to write into */
			   void *USER_DATA    /* User infos passed to WRITE_F */
			   );

/*
 * Dumps the internal structure of a node to a file.
 * Write NODE's structure to OUTPUT_FILE. If WRITE_F != NULL, WRITE_F is used to
 * write NODE's content. 
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t 
 *           & OUTPUT_FILE != NULL
 */
extern void
_gdsl_list_node_dump (const _gdsl_list_node_t NODE,       /* The node to dump */
		      const gdsl_write_func_t WRITE_F,       /* Dump function */
		      FILE *OUTPUT_FILE,            /* The file to write into */
		      void *USER_DATA         /* User infos passed to WRITE_F */
		     );

#ifdef __cplusplus
}
#endif

#endif /* __GDSL_LIST_NODE_H_ */
