/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_list.h
 * $Revision: 1.4 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_list.h,v 1.4 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_LIST_H_
#define _GDSL_LIST_H_

#include "gdsl_types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _gdsl_list *gdsl_list_t;

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a new list.
 * Allocates a new list data structure which name is set to a copy of NAME
 * unless NAME is NULL. 
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The two functions pointers ALLOC_F and FREE_F could be used to respectively, 
 * alloc and free elements in the list.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones: 
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 */
extern gdsl_list_t
gdsl_list_create (const char *NAME,               /* The name of the new list */
		  gdsl_alloc_func_t ALLOC_F,        /* Alloc function pointer */
		  gdsl_free_func_t FREE_F            /* Free function pointer */
		  );

/* 
 * Destroys a list.
 * Destroys all the elements of L by calling L's FREE_F function passed to 
 * gdsl_list_create(). Deallocate L's name and destroy L itself too.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern void 
gdsl_list_destroy (gdsl_list_t L                       /* The list to destroy */
		   );

/* 
 * Flush a list.
 * Destroys all the elements of L by calling L's FREE_F function passed to
 * gdsl_list_create().
 * L is not deallocated itself and L's name is not modified.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern void
gdsl_list_flush (gdsl_list_t L                           /* The list to flush */
		 );

/* 
 * Modify the name of a list.
 * Deallocate the previous L's name if necessary, and change it to a copy of
 * NEW_NAME. 
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_list_t
gdsl_list_set_name (gdsl_list_t L,       /* The list the name must be changed */
		    const char *NEW_NAME                     /* New name of L */
		    );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/* 
 * Gets the name of a list.
 * Returns L's name (and not a copy of it).
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern char *
gdsl_list_get_name (gdsl_list_t L            /* The list to get the name from */
		    );

/* 
 * Returns the cardinality of a list.
 * Returns L's cardinality (noted |L|).
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern ulong
gdsl_list_get_card (gdsl_list_t L     /* The list the cardinality must be get */
		    );

/* 
 * Check if a list is empty.
 * Returns TRUE if L is empty, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern bool
gdsl_list_is_empty (gdsl_list_t L            /* The list the info must be get */
		    );

/*
 * Returns the head of a list.
 * Returns L's head's element if L isn't empty or returns NULL otherwise. The
 * returned element is not removed from L.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_get_head (gdsl_list_t L            /* The list the head must be get */
		    );

/* 
 * Returns the tail of a list.
 * Returns L's tail's element if L isn't empty or returns NULL otherwise. The
 * returned element is not removed from L.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_get_tail (gdsl_list_t L            /* The list the tail must be get */
		    );

/******************************************************************************/
/* Insert functions                                                           */
/******************************************************************************/

/*
 * Inserts a new element at the head of a list.
 * A new element is created using HEAD_VAL with ALLOC_F passed to 
 * gdsl_list_create(). If the returned value is different from NULL, then the new
 * element is placed at L's head. So, the previous L's head is placed after the 
 * new element, after a call to this function.
 * In case of success, the newly inserted element is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_insert_head (gdsl_list_t L,                /* The list to insert in */
		       void *HEAD_VAL    /* Element to insert before L's head */
		       );

/* 
 * Inserts a new element at the tail of a list.
 * A new element is created using TAIL_VAL with ALLOC_F passed to 
 * gdsl_list_create(). If the returned value is different from NULL, then the new
 * element is placed at L's tail. So, the previous L's tail is placed before the
 * new element, after a call to this function. 
 * In case of success, the newly inserted element is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_insert_tail (gdsl_list_t L,                /* The list to insert in */
		       void *TAIL_VAL /* The element to insert after L's tail */
		       );

/******************************************************************************/
/* Remove functions                                                           */
/******************************************************************************/

/*
 * Removes the head from a list.
 * Removes the head from L and returns its element. Returns NULL if L is empty.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_remove_head (gdsl_list_t L               /* The list to remove from */
		       );

/*
 * Removes the tail from a list.
 * Removes the tail from L and returns its element. Returns NULL if L is empty.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_remove_tail (gdsl_list_t L               /* The list to remove from */
		       );

/*
 * Removes a particular element from a list.
 * Try to find an element equal to VAL by using COMP_F. If such an element is
 * found, it is removed from L and returned, if not, NULL is returned. Only the
 * first element assumed to be equal to VAL is removed.
 *
 * Complexity: O( |L| / 2 )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL
 */
extern gdsl_element_t
gdsl_list_remove_by_value (gdsl_list_t L,          /* The list to remove from */
			   gdsl_compare_func_t COMP_F,      /* Comp. function */
			   void *VAL  /* Value used to find element to remove */
			   );

/******************************************************************************/
/* Destroy functions                                                          */
/******************************************************************************/

/* 
 * Destroys the head of a list.
 * Removes the head from L and destroy it using FREE_F passed to 
 * gdsl_list_create().
 * In case of succes, the modified L is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_list_t
gdsl_list_destroy_head (gdsl_list_t L        /* The list to destroy head from */
			);

/* 
 * Destroys the tail of a list.
 * Removes the tail from L and destroy it using FREE_F passed to 
 * gdsl_list_create().
 * In case of succes, the modified L is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_list_t
gdsl_list_destroy_tail (gdsl_list_t L        /* The list to destroy tail from */
			);

/*
 * Destroys a particular element from a list.
 * Try to find an element equal to VAL by using COMP_F. If such an element is
 * found, it is removed from L and deallocated using FREE_F function passed to
 * gdsl_list_create(), then L is returned if the element is found. If no element
 * equals to VAL is found, NULL is returned.
 *
 * Complexity: O( |L| / 2 )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL
 */
extern gdsl_list_t
gdsl_list_destroy_by_value (gdsl_list_t L,        /* The list to destroy from */
			    gdsl_compare_func_t COMP_F,     /* Comp. function */
			    void *VAL/* Value used to find element to destroy */
			    );

/******************************************************************************/
/* Search functions                                                           */
/******************************************************************************/

/*
 * Search for a particular element in a list.
 * Returns the first element of L that is assumed to be equal to VAL, by using
 * COMP_F.
 * Returns NULL if no element equal to VAL is found.
 *
 * Complexity: O( |L| / 2 )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL
 */ 
extern gdsl_element_t
gdsl_list_search_by_value (gdsl_list_t L,            /* The list to search in */
			   gdsl_compare_func_t COMP_F,      /* Comp. function */
			   void *VAL    /* The value used to compare elements */
			   );

/*
 * Search for an element by its position in a list.
 * In case of POS > gdsl_list_get_card( L ) or POS <= 0, returns NULL.
 * In other cases, returns L's POS-th element.
 *
 * Complexity: O( |L| / 2 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_search_by_position (gdsl_list_t L,         /* The list to search in */
			      ulong POS     /* Position of the element to get */
			      );

/*
 * Returns the highest element of a list.
 * Returns the highest element of L, by using COMP_F function.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL
 */
extern gdsl_element_t
gdsl_list_search_max (gdsl_list_t L,                 /* The list to search in */
		      gdsl_compare_func_t COMP_F/* Comparison function to use */
		      );

/*
 * Returns the lowest element of a list.
 * Returns the lowest element of L, by using COMP_F function.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL
 */
extern gdsl_element_t
gdsl_list_search_min (gdsl_list_t L,                 /* The list to search in */
		      gdsl_compare_func_t COMP_F/* Comparison function to use */
		      );

/******************************************************************************/
/* Sort functions                                                             */
/******************************************************************************/

/*
 * Sorts a list.
 * Sorts L using COMP_F to order L's elements. The sorting algorithm used is the
 * 'merge sort'.
 * WARNING: MAX must be an element greather than ALL elements contained into L.
 *
 * Complexity: O( |L| * log( |L| ) )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL 
 *           & L must not contains elements that are equals
 *           & MAX must be higger than *ALL* L's elements
 */
extern gdsl_list_t
gdsl_list_merge_sort (gdsl_list_t L,                      /* The list to sort */
		      gdsl_compare_func_t COMP_F, /* Func to compare elements */
		      gdsl_element_t MAX /* Max element possible *NOT* into L */
		      );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parses a list from head to tail.
 * Parses all L's elements from head to tail. The MAP_F function is launched on
 * each L's elements.
 * If MAP_F returns 0, then gdsl_list_map_forward() stops and returns the actual
 * mapped element. Returns NULL if L's tail is reached before MAP_F returns 0.
 * The USER_DATA argument is passed to MAP_F and isn't modified.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_list_map_forward (gdsl_list_t L,                      /* The list to map */
		       gdsl_map_func_t MAP_F,           /* The parse function */
		       void *USER_DATA                           /* User args */
		       );

/*
 * Parses a list from tail to head.
 * Parses all L's elements from tail to head. The MAP_F function is launched on
 * each L's elements.
 * If MAP_F returns 0, then gdsl_list_map_forward() stops and returns the actual
 * parsed element. Returns NULL if L'head is reached before MAP_F equals 0.
 * The USER_DATA argument is passed to MAP_F and isn't modified.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_list_map_backward (gdsl_list_t L,                     /* The list to map */
			gdsl_map_func_t MAP_F,          /* The parse function */
			void *USER_DATA                          /* User args */
			);

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes all elements of a list to a file.
 * Writes L's nodes's contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() 
 *           & OUTPUT_FILE != NULL
 *           & WRITE_F != NULL
 */
extern void
gdsl_list_write (gdsl_list_t L,                          /* The list to write */
		 gdsl_write_func_t WRITE_F,             /* The write function */
		 FILE *OUTPUT_FILE,                 /* The file to write into */
		 void *USER_DATA       /* User informations passed to WRITE_F */
		 );

/*
 * Writes all elements of a list to a file into XML.
 * Writes L's nodes's contents to OUTPUT_FILE, into XML. 
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_list_write_xml (gdsl_list_t L,                      /* The list to write */
		     gdsl_write_func_t WRITE_F,         /* The write function */
		     FILE *OUTPUT_FILE,             /* The file to write into */
		     void *USER_DATA   /* User informations passed to WRITE_F */
		     );

/*
 * Dumps a list to a file.
 * Dumps the internal data structure of L to OUTPUT_FILE. 
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |L| )
 *
 * Requires: L successfully allocated by gdsl_list_create() 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_list_dump (gdsl_list_t L,                            /* The list to dump */
		gdsl_write_func_t WRITE_F,    /* Function to write L elements */
		FILE *OUTPUT_FILE,                /* The file where to dump L */
		void *USER_DATA        /* User informations passed to WRITE_F */
		);

/******************************************************************************/
/* Cursor specific functions                                                  */
/* WARNING: be carefull to not remove or destroy an element on wich the       */
/*          cursor is positioned!                                             */
/******************************************************************************/

/*
 * Puts the cursor on the head of a list.
 * Sets L's cursor on the head of L. Does nothing if L is empty.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern void
gdsl_list_cursor_move_to_head (gdsl_list_t L   /* The list to move the cursor */
			       );

/*
 * Puts the cursor on the tail of a list.
 * Sets L's cursor on the tail of L. Does nothing if L is empty.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern void
gdsl_list_cursor_move_to_tail (gdsl_list_t L   /* The list to move the cursor */
			       );

/*
 * Moves the cursor forward of a list.
 * Moves L's cursor one node forward (from L's head to L's tail). Does nothing 
 * if L's cursor is already on L's tail.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern void
gdsl_list_cursor_move_forward (gdsl_list_t L   /* The list to move the cursor */
			       );

/*
 * Moves the cursor backward of a list.
 * Moves L's cursor one node backward (from L's tail to L's head.) Does nothing
 * if L's cursor is already on L's head.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern void
gdsl_list_cursor_move_backward (gdsl_list_t L  /* The list to move the cursor */
				);

/*
 * Check if the cursor is on the head of a list.
 * Returns TRUE if the cursor is on L's head, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern bool
gdsl_list_cursor_is_on_head (gdsl_list_t L             /* The list to examine */
			     );

/*
 * Check if the cursor is on the tail of a list.
 * Returns TRUE if the cursor is on L's tail, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern bool
gdsl_list_cursor_is_on_tail (gdsl_list_t L             /* The list to examine */
			     );

/* 
 * Check if the cursor of a list has a successor.
 * Returns TRUE if there exists an element after cursor's one, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern bool 
gdsl_list_cursor_has_succ (gdsl_list_t L               /* The list to examine */
			   );

/* 
 * Check if the cursor of a list has a predecessor.
 * Returns TRUE if there exists an element before cursor's one, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern bool 
gdsl_list_cursor_has_pred (gdsl_list_t L               /* The list to examine */
			   );

/*
 * Sets the element of the cursor of a list.
 * Sets L's cursor element to E. The previous L's cursor element is *NOT* 
 * deallocated. If it must be deallocated, gdsl_list_cursor_get() could be used 
 * to get it in order to free it before. If E == NULL, nothing is done.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create() & E != NULL
 */
extern void
gdsl_list_cursor_set (gdsl_list_t L,         /* The list to modify the cursor */
		      gdsl_element_t E            /* The new cursor's element */
		      );
/*
 * Gets the element of the cursor of a list.
 * Returns L's cursor element.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_cursor_get (gdsl_list_t L             /* The list to get the cursor */
		      );

/*
 * Places the cursor of a list on a particular element E.
 * Research E in L by comparing all L's elements with VAL, using COMP_F.
 * In case of the searched element isn't found, returns NULL.
 * In case of the searched element is found, returns it, and L's cursor is
 * positionned on it.
 *
 * Complexity: O( |L| / 2 )
 *
 * Requires: L successfully allocated by gdsl_list_create() & COMP_F != NULL
 */ 
extern gdsl_element_t
gdsl_list_cursor_search_by_value (gdsl_list_t L,     /* The list to search in */
				  gdsl_compare_func_t COMP_F, /* Comp. funct. */
				  void *VAL /* Value used to compare elements */
				  );

/*
 * Places the cursor of a list on the n-th element.
 * Research for the POS-th element in L.
 * In case of POS > gdsl_list_get_card( l ) or POS <= 0 returns NULL.
 * In other cases, returns the POS-th element of L, and L's cursor is positioned
 * on it.
 *
 * Complexity: O( |L| / 2 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_cursor_search_by_position (gdsl_list_t L,  /* The list to search in */
				     ulong POS  /* Position of element to get */
				     );

/* 
 * Inserts a new element after the cursor in a list.
 * A new element is created using VAL with ALLOC_F passed to gdsl_list_create().
 * If the returned value is different from NULL, then the new element is placed
 * after L's cursor.
 * In case of success, the newly inserted element is returned. If L is empty, 
 * the element is inserted at the head of the list.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_cursor_insert_after (gdsl_list_t L,        /* The list to insert in */
			       void *VAL             /* The element to insert */
			       );
/* 
 * Inserts a new element before the cursor in a list.
 * A new element is created using VAL with ALLOC_F passed to gdsl_list_create().
 * If the returned value is different from NULL, then the new element is placed
 * before L's cursor.
 * In case of success, the newly inserted element is returned. If L is empty,
 * the element is inserted at the head of the list.
 * In case of failure, returns.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_cursor_insert_before (gdsl_list_t L,       /* The list to insert in */
				void *VAL            /* The element to insert */
				);

/*
 * Removes the element after the cursor of a list.
 * Removes the element under the node after the cursor and returns it if it
 * exists.
 * Returns NULL in other cases.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_cursor_remove_after (gdsl_list_t L       /* The list to remove from */
			       );

/*
 * Removes the element before the cursor of a list.
 * Removes the element under the node before the cursor and returns it if it
 * exists.
 * Returns NULL in other cases.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_element_t
gdsl_list_cursor_remove_before (gdsl_list_t L      /* The list to remove from */
				);

/*
 * Destroy the element after the cursor of a list.
 * Exactly the same as gdsl_list_remove_cursor_after() above, but here, the
 * element is deallocated using FREE_F passed to gdsl_list_create() and the 
 * modified L is returned.
 * Returns NULL if no element is present after the cursor.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_list_t
gdsl_list_cursor_destroy_after (gdsl_list_t L     /* The list to destroy from */
				);

/*
 * Destroy the element before the cursor of a list.
 * Exactly the same as gdsl_list_remove_cursor_before() above, but here, the
 * element is deallocated using FREE_F passed to gdsl_list_create() and the 
 * modified L is returned.
 * Returns NULL if no element is present before the cursor.
 *
 * Complexity: O( 1 )
 *
 * Requires: L successfully allocated by gdsl_list_create()
 */
extern gdsl_list_t
gdsl_list_cursor_destroy_before (gdsl_list_t L    /* The list to destroy from */
				 );

#ifdef __cplusplus
}
#endif

#endif /* GDSL_LIST_H_ */
