/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_perm.h
 * $Revision: 1.7 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_perm.h,v 1.7 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_PERM_H_
#define _GDSL_PERM_H_

#include "gdsl_types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct gdsl_perm *gdsl_perm_t;

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a new permutation.
 * Allocates and initializes a new permutation with VALUES and ELEMENTS which 
 * name is set to a copy of NAME unless NAME is NULL.
 * If VALUES is NULL, then the permutation created is the identity permutation.
 * If VALUES is not NULL, it must contains at least N values.
 * If ELEMENTS is NULL, then all contents of the new permutation will be sets to
 * NULL.
 * If ELEMENTS is not NULL, it must contains at least N elements. The elements
 * of the new permutation will be allocated using ALLOC_F.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The two functions pointers ALLOC_F and FREE_F could be used to respectively, 
 * alloc and free elements in the permutation.
 *
 * Complexity: O( N )
 *
 * Requires: N > 0
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones: 
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 */
extern gdsl_perm_t
gdsl_perm_create (const char *NAME,            /* Name of the new permutation */
		  const ulong N,        /* Cardinality of the new permutation */
		  const ulong *VALUES,      /* Values for the new permutation */
		  void **ELEMENTS,        /* Elements for the new permutation */
		  const gdsl_alloc_func_t ALLOC_F,  /* Alloc function pointer */
		  const gdsl_free_func_t FREE_F      /* Free function pointer */
		  );

/*
 * Destroys a permutation.
 * The FREE_F function is used to free elements of P.
 * The permutation P is then deallocated.
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern void
gdsl_perm_destroy (gdsl_perm_t P                /* The permutation to destroy */
		   );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/*
 * Get a permutation's name.
 * Returns the name of the permutation P, and not a copy of it, so the result
 * string must not be deallocated.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern const char *
gdsl_perm_get_name (const gdsl_perm_t P /* */
		    );

/*
 * Gets an element from a permutation.
 * Gets the element at the position N in the permutation P.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 *           & N <= gdsl_perm_get_card (P)
 */
extern ulong
gdsl_perm_get (const gdsl_perm_t P, /* */
	       const ulong N /* */
	       );

/*
 * Gets the cardinality of a permutation.
 * Returns the number of elements of the permutation P.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern ulong
gdsl_perm_get_card (const gdsl_perm_t P /* */
		    );

/*
 * Gets a content from a permutation.
 * Returns the content at the N position from the permutation P.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 *           & N <= gdsl_perm_get_card (P)
 */
extern gdsl_element_t
gdsl_perm_get_content (const gdsl_perm_t P, /* */
		       const ulong N /* */
		       );

/*
 * Counts the number of inversions into a permutation.
 * Computes the number of inversions into the permutation P and returns it.
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern ulong
gdsl_perm_inversions_count (const gdsl_perm_t P     /* The permutation to use */
			    );

/*
 * Counts the number of cycles into a permutation
 * In case of success, computes and returns the number of cycles into the 
 * permutattion P.
 * In case of failure, returns 0 (a given permutation have at least 1 cycle).
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern ulong
gdsl_perm_cycles_count (const gdsl_perm_t P         /* The permutation to use */
			);

/*
 * Counts the number of cycles into a permutation which is in canonical form
 * Computes and returns the number of cycles into the permutation P in its
 * canonical form.
 * WARNING: if P isn't in its canonical form the result is indeterminated
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t in its canonical form
 */
extern ulong
gdsl_perm_canonical_cycles_count (const gdsl_perm_t P     /* The perm. to use */
				  );

/*
 * Gets the next permutation from a permutation.
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_get_next (gdsl_perm_t P /* */
		    );

/*
 * Gets the previous permutation from a permutation.
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_get_prev (gdsl_perm_t P /* */
		    );

/******************************************************************************/
/* Modification functions                                                     */
/******************************************************************************/

/*
 * Change a permutation's name.
 * Modify the name of the permutation P to NEW_NAME and not to a copy of it. The
 * previous name is deallocated before.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_set_name (gdsl_perm_t P,  /* */
		    const char *NEW_NAME /* */
		    );
/*
 * Sets an element into a permutation.
 * Sets the elements in the positio N to E into the permutation P.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 *           & N <= |P|
 */
extern void
gdsl_perm_set (gdsl_perm_t P, /* */
	       const ulong N, /* */
	       const ulong E /* */
	       );

/*
 * Sets a content into a permutation.
 * The previous N-th content of P is *NOT* deallocated, it's up to you to do
 * that, by calling gdsl_perm_get_content( P, N ) to get and free it yourself.
 *
 * Complexity: O( 1 )
 *
 * Requires: P must be a valid gdsl_perm_t
 *           & N <= |P|
 */
extern gdsl_element_t
gdsl_perm_set_content (gdsl_perm_t P,  /* */
		       const ulong N,  /* */
		       void *E /* */
		       );

/******************************************************************************/
/* Operations functions                                                       */
/******************************************************************************/

/*
 * Multiplies two permutations.
 * Computes ALPHA x BETA, and puts the result into RESULT.
 * In case of succes, returns the modified RESULT.
 * In case of failure, returns NULL.
 *
 * Complexity: O( |ALPHA| )
 *
 * Requires: RESULT != NULL
 *           & |RESULT| == |ALPHA| == |BETA| 
 *           & RESULT, ALPHA and BETA must be valids gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_mul (gdsl_perm_t RESULT,
	       const gdsl_perm_t ALPHA,
	       const gdsl_perm_t BETA
	       );
  
/*
 * Converts a permutation to its canonical representation.
 * Converts the permutation P to its canonical representation using the Foata's
 * bijection.
 * In case of success, returns the modified permutation.
 * In case of failure, returns NULL.
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_to_canonical (gdsl_perm_t P /* */
			);

/*
 * Converts a canonical permutation to its classic form.
 * Converts the permutation P from its canonical representation to its normal
 * representation using the inverse Foata's bijection.
 * In case of success, returns the modified permutation.
 * In case of failure, returns NULL.
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_canonical_to_perm (gdsl_perm_t P /* */
			     );

/*
 * Inverse a permutation.
 * In case of success, computes and returns the inverse permutation P.
 * In case of failure, returns NULL.
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t
 *
 */
extern gdsl_perm_t
gdsl_perm_inverse (gdsl_perm_t P                 /* The permutation to invert */
		   );

/*
 * Reverses a permutation.
 * Computes the reverse permutation P.
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t
 *
 */
extern void
gdsl_perm_reverse (gdsl_perm_t P                /* The permutation to reverse */
		   );

/*
 * Shuffles a permutation.
 * In case of success, shuffles and returns the permutation P using the 
 * inversion tab bijection.
 * In case of failure, returns NULL.
 *
 * Complexity: O( |P| )
 *
 * Requires: P must be a valid gdsl_perm_t
 */
extern gdsl_perm_t
gdsl_perm_randomize (gdsl_perm_t P              /* The permutation to shuffle */
		     );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * 
 * Complexity: O( |P| )
 *
 * Requires: P successfully allocated by gdsl_perm_create() or by
 *           gdsl_perm_rand_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_perm_map_forward (const gdsl_perm_t P,         /* The permutation to map */
		       const gdsl_map_func_t MAP_F,
		       void *USER_DATA
		       );

/*
 *
 * Complexity: O( |P| )
 * 
 * Requires: P successfully allocated by gdsl_perm_create() or by
 *           gdsl_perm_rand_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_perm_map_backward (const gdsl_perm_t P,        /* The permutation to map */
			const gdsl_map_func_t MAP_F,
			void *USER_DATA
			);

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes content of a permutation to a file
 *
 * Complexity: O( |P| )
 */
extern void
gdsl_perm_write (const gdsl_perm_t P,
		 const gdsl_write_func_t WRITE_F, 
		 FILE *OUTPUT_FILE, 
		 void *USER_DATA
		 );
  
/*
 *
 * Complexity: O( |P| )
 */
extern void
gdsl_perm_write_xml (const gdsl_perm_t P,
		     const gdsl_write_func_t WRITE_F, 
		     FILE *OUTPUT_FILE, 
		     void *USER_DATA
		     );

/*
 *
 * Complexity: O( |P| )
 */
extern void
gdsl_perm_dump (const gdsl_perm_t P,
		const gdsl_write_func_t WRITE_F, 
		FILE *OUTPUT_FILE, 
		void *USER_DATA
		);
  
#ifdef __cplusplus
}
#endif

#endif /* _GDSL_PERM_H_ */
