/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_queue.h
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_queue.h,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_QUEUE_H_
#define _GDSL_QUEUE_H_

#include <stdio.h>

#include "gdsl_types.h"

typedef struct _gdsl_queue *gdsl_queue_t;

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a new queue.
 * Creates a new allocated queue data structure which name is set to a copy of
 * GDSL_QUEUE_NAME unless GDSL_QUEUE_NAME is NULL.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The two functions pointers ALLOC_F and FREE_F could be used to respectively, 
 * alloc and free elements in the queue.
 *
 * Complexity: O( 1 )
 *
 * Requires: ALLOC_F & FREE_F must be defined as explained in gdsl_types.h
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones:
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 */
extern gdsl_queue_t
gdsl_queue_create (const char *NAME,             /* The name of the new queue */
		   gdsl_alloc_func_t ALLOC_F,       /* Alloc function pointer */
		   gdsl_free_func_t FREE_F           /* Free function pointer */
		   );

/*
 * Destroy a queue.
 * Destroy all the elements of Q by calling Q's FREE_F function passed to
 * gdsl_queue_create() function. Deallocate Q's name and destroy Q itself too.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern void 
gdsl_queue_destroy (gdsl_queue_t Q                    /* The queue to destroy */
		    );

/*
 * Flush a queue.
 * Same as above but Q is not destroyed (only the elements are removed and
 * free-ed by calling Q's FREE_F function passed to gdsl_queue_create()).
 * Q's name is not modified.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern void
gdsl_queue_flush (gdsl_queue_t Q                        /* The queue to flush */
		  );

/*
 * Sets the name of a queue.
 * Deallocate the previous Q's name if exists, and change it to a copy of
 * GDSL_QUEUE_NEW_NAME.
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern gdsl_queue_t
gdsl_queue_set_name (gdsl_queue_t Q,    /* The queue the name must be changed */
		     const char *NEW_NAME                    /* New name of Q */
		     );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/*
 * Gets the name of a queue.
 * Returns Q's name (and not a copy of it).
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern const char *
gdsl_queue_get_name (gdsl_queue_t Q         /* The queue to get the name from */
		     );

/* 
 * Returns the cardinality of a queue.
 * Returns Q's cardinality.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern ulong
gdsl_queue_get_card (gdsl_queue_t Q  /* The queue to get the cardinality from */
		     );

/*
 * Check if a queue is empty.
 * Returns TRUE if Q is empty, FALSE otherwise.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern bool 
gdsl_queue_is_empty (gdsl_queue_t Q         /* The queue to get the info from */
		     );

/*
 * Gets the head of a queue.
 * Returns Q's head *WITHOUT* removing it from Q if Q ins't empty, or returns
 * NULL otherwise.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern gdsl_element_t
gdsl_queue_get_head (gdsl_queue_t Q             /* The queue to get head from */
		     );

/*
 * Gets the tail of a queue.
 * Returns Q's tail *WITHOUT* removing it from Q if Q ins't empty, or returns 
 * NULL otherwise.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern gdsl_element_t
gdsl_queue_get_tail (gdsl_queue_t Q             /* The queue to get tail from */
		     );

/******************************************************************************/
/* Insert functions                                                           */
/******************************************************************************/

/*
 * Put an element into a queue.
 * Alloc and put into Q a new element using E and the ALLOC_F function passed to 
 * gdsl_queue_create().
 * In case of success, returns the newly enqueued element.
 * In case of failure, returns NULL.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern gdsl_element_t
gdsl_queue_put (gdsl_queue_t Q,                        /* The queue to put in */
		void *E                            /* The element to put in Q */
		);

/******************************************************************************/
/* Remove functions                                                           */
/******************************************************************************/

/*
 * Get an element from a queue.
 * Get and return Q's head element if Q isn't empty, returns NULL otherwise.
 * The element must be free-ed after use by FREE_F.
 * Requires: Q successfully allocated by gdsl_queue_create()
 */
extern gdsl_element_t
gdsl_queue_get (gdsl_queue_t Q               /* The queue to get element from */
		);

/******************************************************************************/
/* Search functions                                                           */
/******************************************************************************/

/*
 * Search for a particular element in a queue.
 * Returns the first element of Q that is assumed to be equal to KEY, by using
 * COMP_F.
 * Returns NULL if no element equal to KEY is found.
 * Requires: Q successfully allocated by gdsl_queue_create() 
 */
extern gdsl_element_t
gdsl_queue_search (gdsl_queue_t Q,                  /* The queue to search in */
		   gdsl_compare_func_t COMP_F,  /* Comparison function to use */
		   void *KEY          /* The key used to compare Q's elements */
		   );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse a queue from tail to head.
 * Parse all Q's elements from tail to head of Q. The MAP_F function is 
 * launched on each element of Q.
 * If MAP_F returns 0, then gdsl_queue_map() will stops and returns the actual
 * parsed element. Returns NULL if Q's head is reached before MAP_F returns 0.
 * The USER_INFOS argument is passed to MAP_F and isn't modified.
 * Requires: Q successfully allocated by gdsl_queue_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_queue_map (gdsl_queue_t Q,                           /* The queue to map */
		gdsl_map_func_t MAP_F,                  /* The parse function */
		void *USER_INFOS                                 /* User args */
		);

/******************************************************************************/
/* Output functions                                                           */
/******************************************************************************/

/*
 * Writes all elements of a queue to a file.
 * Writes Q's nodes's contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 * Requires: Q successfully allocated by gdsl_queue_create()
 *           & OUTPUT_FILE != NULL
 *           & WRITE_F != NULL
 */
extern void
gdsl_queue_write (gdsl_queue_t Q,                       /* The queue to write */
		  gdsl_write_func_t WRITE_F,            /* The write function */
		  FILE *OUTPUT_FILE,                /* The file to write into */
		  void *USER_DATA      /* User informations passed to WRITE_F */
		  );

/*
 * Writes all elements of a queue to a file into XML language.
 * Writes Q's nodes's contents to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 * Requires: Q successfully allocated by gdsl_queue_create() 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_queue_write_xml (gdsl_queue_t Q,                   /* The queue to write */
		      gdsl_write_func_t WRITE_F,        /* The write function */
		      FILE *OUTPUT_FILE,            /* The file to write into */
		      void *USER_DATA  /* User informations passed to WRITE_F */
		      );

/*
 * Dumps the whole queue to a file.
 * Dumps Q to OUTPUT_FILE. 
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 * Requires: Q successfully allocated by gdsl_queue_create()
 *           & OUTPUT_FILE != NULL
 */
extern void 
gdsl_queue_dump (gdsl_queue_t Q,                         /* The queue to dump */
		 gdsl_write_func_t WRITE_F, /* Func. used to write Q elements */
		 FILE *OUTPUT_FILE,               /* The file where to dump Q */
		 void *USER_DATA       /* User informations passed to WRITE_F */
		 );

#endif /* _GDSL_QUEUE_H_ */
