/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_stack.h
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_stack.h,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_STACK_H_
#define _GDSL_STACK_H_

#include <stdio.h>

#include "gdsl_types.h"

typedef struct _gdsl_stack *gdsl_stack_t;

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a new stack.
 * Creates a new allocated stack data structure which name is set to a copy of
 * NAME unless GDSL_STACK_NAME is NULL.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The two functions pointers ALLOC_F and FREE_F could be used to respectively, 
 * alloc and free elements in the stack.
 *
 * Complexity: O( 1 )
 *
 * Requires: ALLOC_F & FREE_F must be defined as explained in gdsl_types.h
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones:
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 */
extern gdsl_stack_t
gdsl_stack_create (const char *NAME,             /* The name of the new stack */
		   gdsl_alloc_func_t ALLOC_F,       /* Alloc function pointer */
		   gdsl_free_func_t FREE_F           /* Free function pointer */
		   );

/*
 * Destroy a stack.
 * Destroy all the elements of S by calling S's FREE_F function passed to
 * gdsl_stack_create() function. Deallocate S's name and destroy S itself too.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern void 
gdsl_stack_destroy (gdsl_stack_t S                    /* The stack to destroy */
		    );

/*
 * Flush a stack.
 * Same as above but S is not destroyed (only the elements are removed and
 * free-ed by calling S's FREE_F function passed to gdsl_stack_create()).
 * In case of success, returns the modified S.
 * In case of failure, returns NULL if _NO_EXCEPTIONS_ is defined, or raise
 * gdsl_stack_default_EXCEPTION otherwise.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern gdsl_stack_t
gdsl_stack_flush (gdsl_stack_t S                        /* The stack to flush */
		  );

/*
 * Sets the name of a stack.
 * Deallocate the previous S's name if exists, and change it to a copy of
 * NEW_NAME.
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL if _NO_EXCEPTIONS_ is defined, raises
 * gdsl_stack_default_EXCEPTION otherwise.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern gdsl_stack_t
gdsl_stack_set_name (gdsl_stack_t S,    /* The stack the name must be changed */
		     const char *NEW_NAME                    /* New name of S */
		     );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/*
 * Gets the name of a stack.
 * Returns S's name (and not a copy of it).
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern const char *
gdsl_stack_get_name (gdsl_stack_t S         /* The stack to get the name from */
		     );

/*
 * Check if a stack is empty.
 * Returns TRUE if S is empty, FALSE otherwise.
 * Requires: S successfully allocated by gdsl_stck_create()
 */
extern bool 
gdsl_stack_is_empty (gdsl_stack_t S         /* The stack to get the info from */
		     );

/*
 * Gets element at the top of a stack.
 * Returns S's top *WITHOUT* removing it from S if S ins't empty, or returns
 * NULL otherwise.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern gdsl_element_t
gdsl_stack_get_top (gdsl_stack_t S               /* The stack to get top from */
		    );

/*
 * Gets element at the bottom of a stack.
 * Returns S's tail *WITHOUT* removing it from S if S ins't empty, or returns 
 * NULL otherwise.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern gdsl_element_t
gdsl_stack_get_bottom (gdsl_stack_t S         /* The stack to get bottom from */
		       );

/******************************************************************************/
/* Insert functions                                                           */
/******************************************************************************/

/*
 * Push an element into a stack.
 * Alloc and push into S a new element using E and the ALLOC_F function passed 
 * to gdsl_stack_create().
 * In case of success, returns the newly enqueued element.
 * In case of failure, and if _NO_EXCEPTIONS_ is defined returns NULL, raises
 * gdsl_stack_default_EXCEPTION otherwise.
 * Requires: Q successfully allocated by gdsl_stack_create()
 */
extern gdsl_element_t
gdsl_stack_push (gdsl_stack_t S,                      /* The stack to push in */
		 void *E                          /* The element to push in S */
		 );

/******************************************************************************/
/* Remove functions                                                           */
/******************************************************************************/

/*
 * Pop an element from a stack.
 * Pop and return S's top element if S isn't empty, returns NULL otherwise.
 * The element must be free-ed after use by FREE_F.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern gdsl_element_t
gdsl_stack_pop (gdsl_stack_t S               /* The stack to pop element from */
		);

/******************************************************************************/
/* Search functions                                                           */
/******************************************************************************/

/*
 * Search for a particular element in a stack.
 * Returns the first element of S that is assumed to be equal to KEY, by using
 * COMP_F.
 * Returns NULL if no element equal to KEY is found.
 * Requires: S successfully allocated by gdsl_stack_create()
 */
extern gdsl_element_t
gdsl_stack_search (gdsl_stack_t S,                  /* The stack to search in */
		   gdsl_compare_func_t COMP_F,  /* Comparison function to use */
		   void *KEY          /* The key used to compare S's elements */
		   );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse from bottom to top a stack.
 * Parse all S's elements from bottom to top. The MAP_F function is launched on
 * each element of S.
 * If MAP_F returns 0, then gdsl_stack_map() stops and returns the actual parsed
 * element. Returns NULL if S's top is reached before MAP_F returns 0.
 * The USER_INFOS argument is passed to MAP_F and isn't modified.
 * Requires: S successfully allocated by gdsl_stack_create() and MAP_F != NULL
 */
extern void 
gdsl_stack_map (gdsl_stack_t S,                           /* The stack to map */
		gdsl_map_func_t F,                      /* The parse function */
		void *USER_INFOS                                 /* User args */
		);

/******************************************************************************/
/* Output functions                                                           */
/******************************************************************************/

/*
 * Writes all elements of a stack to a file.
 * Writes S's elements to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 * Requires: S successfully allocated by gdsl_stack_create() 
 *           & OUTPUT_FILE != NULL
 *           & WRITE_F != NULL
 */
extern void
gdsl_stack_write (gdsl_stack_t S,                       /* The stack to write */
		  gdsl_write_func_t WRITE_F,            /* The write function */
		  FILE *OUTPUT_FILE,                /* The file to write into */
		  void *USER_DATA      /* User informations passed to WRITE_F */
		  );

/*
 * Writes all elements of a stack to a file into XML language.
 * Writes S's elements to OUTPUT_FILE, into XML language. 
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 * Requires: S successfully allocated by gdsl_stack_create() 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_stack_write_xml (gdsl_stack_t S,                   /* The stack to write */
		      gdsl_write_func_t WRITE_F,        /* The write function */
		      FILE *OUTPUT_FILE,            /* The file to write into */
		      void *USER_DATA  /* User informations passed to WRITE_F */
		      );

/*
 * Dumps the whole stack to a file.
 * Dumps S to OUTPUT_FILE. 
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 * Requires: S successfully allocated by gdsl_stack_create() 
 *           & OUTPUT_FILE != NULL
 */
extern void 
gdsl_stack_dump (gdsl_stack_t S,                         /* The stack to dump */
		 gdsl_write_func_t WRITE_F, /* Function used to write content */
		 FILE *OUTPUT_FILE,               /* The file where to dump S */
		 void *USER_DATA       /* User informations passed to WRITE_F */
		 );

#endif /* _GDSL_STACK_H_ */
